/*******************************************************************************
 * Copyright (c) 2004, 2006 svnClientAdapter project and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 *     svnClientAdapter project committers - initial API and implementation
 ******************************************************************************/
package org.tigris.subversion.svnclientadapter;

/**
 * Subversion related global constants
 *
 */
public class SVNConstants {
	
	/** a "entries" workingCopy metadata directory name */
    public static final String SVN_ENTRIES = "wc.db";
    
	/** a "dir-props" workingCopy metadata directory name */
    public static final String SVN_DIRPROPS = "dir-props";
    
	/** a "props" workingCopy metadata directory name */
    public static final String SVN_PROPS = "props";
}
